==> dnet-mapreduce-jobs-0.0.4.1-20141022.164838-23-sources.jar: eu/dnetlib/data/mapreduce/hbase/dedup/DedupFindRootsMapper.java <==
package eu.dnetlib.data.mapreduce.hbase.dedup;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.Map;

import org.apache.hadoop.hbase.client.Put;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.io.ImmutableBytesWritable;
import org.apache.hadoop.hbase.mapreduce.TableMapper;
import org.apache.hadoop.hbase.util.Bytes;

import com.google.protobuf.InvalidProtocolBufferException;

import eu.dnetlib.data.mapreduce.util.DedupUtils;
import eu.dnetlib.data.proto.DedupProtos.Dedup;
import eu.dnetlib.data.proto.KindProtos.Kind;
import eu.dnetlib.data.proto.OafProtos.Oaf;
import eu.dnetlib.data.proto.OafProtos.OafRel.Builder;
import eu.dnetlib.data.proto.TypeProtos.Type;
import eu.dnetlib.data.transform.xml.AbstractDNetOafXsltFunctions;
import eu.dnetlib.pace.util.DedupConfig;
import eu.dnetlib.pace.util.DedupConfigLoader;

public class DedupFindRootsMapper extends TableMapper<ImmutableBytesWritable, Put> {

	private static final boolean WRITE_TO_WAL = false;

	private DedupConfig dedupConf;

	@Override
	protected void setup(final Context context) throws IOException, InterruptedException {
		dedupConf = DedupConfigLoader.load(context.getConfiguration().get("dedup.wf.conf"));
		System.out.println("dedup findRoots mapper\nwf conf: " + dedupConf.toString());
	}

	@Override
	protected void map(final ImmutableBytesWritable rowkey, final Result value, final Context context) throws IOException, InterruptedException {
		// System.out.println("Find root mapping: " + new String(rowkey.copyBytes()));

		Type type = Type.valueOf(dedupConf.getEntityType());
		Map<byte[], byte[]> similarRels = value.getFamilyMap(DedupUtils.getSimilarityCFBytes(type));

		if ((similarRels != null) && !similarRels.isEmpty()) {
			ByteBuffer min = findMin(ByteBuffer.wrap(rowkey.get()), similarRels.keySet());

			byte[] row = rowkey.copyBytes();
			byte[] root = DedupUtils.newIdBytes(min, dedupConf.getDedupRun());

			// System.out.println("Found root: " + new String(root));

			emitDedupRel(context, DedupUtils.getDedupCF_mergedInBytes(type), row, root, buildRel(row, root, Dedup.RelName.isMergedIn));
			emitDedupRel(context, DedupUtils.getDedupCF_mergesBytes(type), root, row, buildRel(root, row, Dedup.RelName.merges));

			context.getCounter(dedupConf.getEntityType(), "dedupRel (x2)").increment(1);

			// marks the original body deleted
			emitBody(context, row, value.getValue(Bytes.toBytes(dedupConf.getEntityType()), DedupUtils.BODY_B));

		} else {
			context.getCounter(dedupConf.getEntityType(), "row not in similarity mesh").increment(1);
		}
	}

	private ByteBuffer findMin(ByteBuffer min, final Iterable<byte[]> keys) {
		for (byte[] q : keys) {
			ByteBuffer iq = ByteBuffer.wrap(q);
			if (min.compareTo(iq) > 0) {
				min = iq;
			}
		}
		return min;
	}

	private void emitBody(final Context context, final byte[] row, final byte[] body) throws InvalidProtocolBufferException, IOException, InterruptedException {
		if (body == null) {
			context.getCounter(dedupConf.getEntityType(), "missing body").increment(1);
			System.err.println("missing body: " + new String(row));
			return;
		}
		final Oaf prototype = Oaf.parseFrom(body);

		if (prototype.getDataInfo().getDeletedbyinference()) {
			context.getCounter(dedupConf.getEntityType(), "bodies already deleted").increment(1);
		} else {
			Oaf.Builder oafRoot = Oaf.newBuilder(prototype);
			oafRoot.getDataInfoBuilder().setDeletedbyinference(true).setInferred(true).setInferenceprovenance("dedup");
			byte[] family = Bytes.toBytes(dedupConf.getEntityType());
			Put put = new Put(row).add(family, DedupUtils.BODY_B, oafRoot.build().toByteArray());
			put.setWriteToWAL(WRITE_TO_WAL);
			context.write(new ImmutableBytesWritable(row), put);
			context.getCounter(dedupConf.getEntityType(), "bodies marked deleted").increment(1);
		}
	}

	private byte[] buildRel(final byte[] from, final byte[] to, final Dedup.RelName relClass) {
		Builder oafRel = DedupUtils.getDedup(dedupConf, new String(from), new String(to), relClass);
		Oaf oaf =
				Oaf.newBuilder().setKind(Kind.relation).setTimestamp(System.currentTimeMillis())
				.setDataInfo(AbstractDNetOafXsltFunctions.getDataInfo(null, "", "0.8", false, true).setInferenceprovenance("dedup")).setRel(oafRel)
						.build();
		return oaf.toByteArray();
	}

	private void emitDedupRel(final Context context, final byte[] cf, final byte[] from, final byte[] to, final byte[] value) throws IOException,
	InterruptedException {
		Put put = new Put(from).add(cf, to, value);
		put.setWriteToWAL(WRITE_TO_WAL);
		context.write(new ImmutableBytesWritable(from), put);
	}

}

==> dnet-mapreduce-jobs-0.0.4-20140711.165619-25-sources.jar: eu/dnetlib/data/mapreduce/hbase/dedup/DedupFindRootsMapper.java <==
package eu.dnetlib.data.mapreduce.hbase.dedup;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.Map;

import org.apache.hadoop.hbase.client.Put;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.io.ImmutableBytesWritable;
import org.apache.hadoop.hbase.mapreduce.TableMapper;
import org.apache.hadoop.hbase.util.Bytes;

import com.google.protobuf.InvalidProtocolBufferException;

import eu.dnetlib.data.mapreduce.util.DedupUtils;
import eu.dnetlib.data.proto.DedupProtos.Dedup;
import eu.dnetlib.data.proto.KindProtos.Kind;
import eu.dnetlib.data.proto.OafProtos.Oaf;
import eu.dnetlib.data.proto.OafProtos.OafRel.Builder;
import eu.dnetlib.data.proto.TypeProtos.Type;
import eu.dnetlib.data.transform.xml.AbstractDNetOafXsltFunctions;
import eu.dnetlib.pace.util.DedupConfig;
import eu.dnetlib.pace.util.DedupConfigLoader;

public class DedupFindRootsMapper extends TableMapper<ImmutableBytesWritable, Put> {

	private static final boolean WRITE_TO_WAL = false;

	private DedupConfig dedupConf;

	@Override
	protected void setup(final Context context) throws IOException, InterruptedException {
		dedupConf = DedupConfigLoader.load(context.getConfiguration().get("dedup.wf.conf"));
		System.out.println("dedup findRoots mapper\nwf conf: " + dedupConf.toString());
	}

	@Override
	protected void map(final ImmutableBytesWritable rowkey, final Result value, final Context context) throws IOException, InterruptedException {
		// System.out.println("Find root mapping: " + new String(rowkey.copyBytes()));

		Type type = Type.valueOf(dedupConf.getEntityType());
		Map<byte[], byte[]> similarRels = value.getFamilyMap(DedupUtils.getSimilarityCFBytes(type));

		if ((similarRels != null) && !similarRels.isEmpty()) {
			ByteBuffer min = findMin(ByteBuffer.wrap(rowkey.get()), similarRels.keySet());

			byte[] row = rowkey.copyBytes();
			byte[] root = DedupUtils.newIdBytes(min, dedupConf.getDedupRun());

			System.out.println("Found root: " + new String(root));

			emitDedupRel(context, DedupUtils.getDedupCF_mergedInBytes(type), row, root, buildRel(row, root, Dedup.RelName.isMergedIn));
			emitDedupRel(context, DedupUtils.getDedupCF_mergesBytes(type), root, row, buildRel(root, row, Dedup.RelName.merges));

			context.getCounter(dedupConf.getEntityType(), "dedupRel (x2)").increment(1);

			// marks the original body deleted
			emitBody(context, row, value.getValue(Bytes.toBytes(dedupConf.getEntityType()), DedupUtils.BODY_B));

		} else {
			context.getCounter(dedupConf.getEntityType(), "row not in similarity mesh").increment(1);
		}
	}

	private ByteBuffer findMin(ByteBuffer min, final Iterable<byte[]> keys) {
		for (byte[] q : keys) {
			ByteBuffer iq = ByteBuffer.wrap(q);
			if (min.compareTo(iq) > 0) {
				min = iq;
			}
		}
		return min;
	}

	private void emitBody(final Context context, final byte[] row, final byte[] body) throws InvalidProtocolBufferException, IOException, InterruptedException {
		if (body == null) {
			context.getCounter(dedupConf.getEntityType(), "missing body").increment(1);
			System.err.println("missing body: " + new String(row));
			return;
		}
		final Oaf prototype = Oaf.parseFrom(body);

		if (prototype.getDataInfo().getDeletedbyinference()) {
			context.getCounter(dedupConf.getEntityType(), "bodies already deleted").increment(1);
		} else {
			Oaf.Builder oafRoot = Oaf.newBuilder(prototype);
			oafRoot.getDataInfoBuilder().setDeletedbyinference(true).setInferred(true).setInferenceprovenance("dedup");
			byte[] family = Bytes.toBytes(dedupConf.getEntityType());
			Put put = new Put(row).add(family, DedupUtils.BODY_B, oafRoot.build().toByteArray());
			put.setWriteToWAL(WRITE_TO_WAL);
			context.write(new ImmutableBytesWritable(row), put);
			context.getCounter(dedupConf.getEntityType(), "bodies marked deleted").increment(1);
		}
	}

	private byte[] buildRel(final byte[] from, final byte[] to, final Dedup.RelName relClass) {
		Builder oafRel = DedupUtils.getDedup(dedupConf, new String(from), new String(to), relClass);
		Oaf oaf =
				Oaf.newBuilder().setKind(Kind.relation).setTimestamp(System.currentTimeMillis())
						.setDataInfo(AbstractDNetOafXsltFunctions.getDataInfo(null, "0.8", false, true).setInferenceprovenance("dedup")).setRel(oafRel).build();
		return oaf.toByteArray();
	}

	private void emitDedupRel(final Context context, final byte[] cf, final byte[] from, final byte[] to, final byte[] value) throws IOException,
			InterruptedException {
		Put put = new Put(from).add(cf, to, value);
		context.write(new ImmutableBytesWritable(from), put);
	}

}
